/*
 * Zorin App Grid: Show the App Grid button in the panel.
 *
 * Copyright (C) 2019 Zorin OS Technologies Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

const Clutter = imports.gi.Clutter;
const GLib = imports.gi.GLib;
const Lang = imports.lang;
const Mainloop = imports.mainloop;
const St = imports.gi.St;
const Atk = imports.gi.Atk;
const Main = imports.ui.main;
const DND = imports.ui.dnd;
const PanelMenu = imports.ui.panelMenu;
const Panel = imports.ui.panel;
const Gettext = imports.gettext.domain('gnome-shell');
const _ = Gettext.gettext;


var ActivitiesButton = new Lang.Class({
    Name: 'ActivitiesButton',
    Extends: PanelMenu.Button,

    _init() {
        this.parent(0.0, null, true);
        this.actor.accessible_role = Atk.Role.TOGGLE_BUTTON;

        this.actor.name = 'panelActivities';

        /* Translators: If there is no suitable word for "Activities"
           in your language, you can use the word for "Overview". */
        this._label = new St.Label({ text: _("Activities"),
                                     y_align: Clutter.ActorAlign.CENTER });
        this.actor.add_actor(this._label);

        this.actor.label_actor = this._label;

        this.actor.connect('captured-event', this._onCapturedEvent.bind(this));
        this.actor.connect_after('key-release-event', this._onKeyRelease.bind(this));

        this._overviewShowingId = Main.overview.connect('showing', () => {
            if (Main.overview.viewSelector._activePage == Main.overview.viewSelector._workspacesPage) {
                this.actor.add_style_pseudo_class('overview');
                this.actor.add_accessible_state (Atk.StateType.CHECKED);
            }
        });
        this._overviewHidingId = Main.overview.connect('hiding', () => {
            this.actor.remove_style_pseudo_class('overview');
            this.actor.remove_accessible_state (Atk.StateType.CHECKED);
        });

        this._overviewPageChangedId = Main.overview.viewSelector.connect('page-changed', () => {
            if (Main.overview.viewSelector._activePage == Main.overview.viewSelector._workspacesPage) {
                this.actor.add_style_pseudo_class('overview');
                this.actor.add_accessible_state (Atk.StateType.CHECKED);
            } else {
                this.actor.remove_style_pseudo_class('overview');
                this.actor.remove_accessible_state (Atk.StateType.CHECKED);
            }
        });

        this._xdndTimeOut = 0;
    },

    _toggleOverview() {
        if (Main.overview.isDummy)
            return;

        if (Main.overview.visible) {
            if (Main.overview.viewSelector._activePage != Main.overview.viewSelector._workspacesPage) {
                Main.overview.viewSelector._showPage(Main.overview.viewSelector._workspacesPage);
                Main.overview.viewSelector._showAppsButton.checked = false;
            } else {
                Main.overview.hide();
            }
        } else {
            Main.overview.show();
        }
    },

    handleDragOver(source, actor, x, y, time) {
        if (source != Main.xdndHandler)
            return DND.DragMotionResult.CONTINUE;

        if (this._xdndTimeOut != 0)
            Mainloop.source_remove(this._xdndTimeOut);
        this._xdndTimeOut = Mainloop.timeout_add(Panel.BUTTON_DND_ACTIVATION_TIMEOUT, () => {
            this._xdndToggleOverview(actor);
        });
        GLib.Source.set_name_by_id(this._xdndTimeOut, '[gnome-shell] this._xdndToggleOverview');

        return DND.DragMotionResult.CONTINUE;
    },

    _onCapturedEvent(actor, event) {
        if (event.type() == Clutter.EventType.BUTTON_PRESS ||
            event.type() == Clutter.EventType.TOUCH_BEGIN) {
            if (!Main.overview.shouldToggleByCornerOrButton())
                return Clutter.EVENT_STOP;
        }
        return Clutter.EVENT_PROPAGATE;
    },

    _onEvent(actor, event) {
        this.parent(actor, event);

        if (event.type() == Clutter.EventType.TOUCH_END ||
            event.type() == Clutter.EventType.BUTTON_RELEASE)
            if (Main.overview.shouldToggleByCornerOrButton())
                this._toggleOverview();

        return Clutter.EVENT_PROPAGATE;
    },

    _onKeyRelease(actor, event) {
        let symbol = event.get_key_symbol();
        if (symbol == Clutter.KEY_Return || symbol == Clutter.KEY_space) {
            if (Main.overview.shouldToggleByCornerOrButton())
                this._toggleOverview();
        }
        return Clutter.EVENT_PROPAGATE;
    },

    _xdndToggleOverview(actor) {
        let [x, y, mask] = global.get_pointer();
        let pickedActor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, x, y);

        if (pickedActor == this.actor && Main.overview.shouldToggleByCornerOrButton())
            this._toggleOverview();

        Mainloop.source_remove(this._xdndTimeOut);
        this._xdndTimeOut = 0;
        return GLib.SOURCE_REMOVE;
    },

    destroy() {
        Main.overview.disconnect(this._overviewShowingId);
        Main.overview.disconnect(this._overviewHidingId);
        Main.overview.viewSelector.disconnect(this._overviewPageChangedId);
        
        this.parent();
    }
});
