/*
 * Zorin App Grid: Show the App Grid button in the panel.
 *
 * Copyright (C) 2019 Zorin OS Technologies Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const Lang = imports.lang;
const Meta = imports.gi.Meta;
const IconGrid = imports.ui.iconGrid;
const St = imports.gi.St;
const Main = imports.ui.main;
const Gettext = imports.gettext.domain('gnome-shell');
const _ = Gettext.gettext;

const Me = imports.misc.extensionUtils.getCurrentExtension();


// Menu Button icon
const menu_icon = Me.path + '/zorin-icon-symbolic.svg';

// Menu Button padding variable
const MINIMUM_PADDING = 4;

// Base Menu Button Icon Sizes
const baseMenuButtonIconSizes = [ 16, 22, 24, 32, 48, 64, 96, 128 ];


var AppGridButton = new Lang.Class({
    Name: 'AppGridButton',
    Extends: St.Button,

    _init(panelActor) {
        this._panelActor = panelActor;

        this.parent({ style_class: 'panel-button',
                      track_hover: true,
                      can_focus: true,
                      toggle_mode: true });

        this.actor = this;
        this._delegate = this;
        this.showAppsButton = this;

        this.actor.name = 'showApps';
        this.actor.set_accessible_name(_("Show Applications"));

        this.connect('style-changed', this._onStyleChanged.bind(this));
        this.connect('destroy', this._onDestroy.bind(this));

        this.showAppsButton.connect('notify::checked', Lang.bind(this, this._onShowAppsButtonToggled));
        this.showAppsButton.checked = Main.overview.viewSelector._showAppsButton.checked;

        this._iconSize = 32;

        this._icon = new St.Icon({
            gicon: Gio.icon_new_for_string(menu_icon),
            icon_size: this._iconSize,
            style_class: 'popup-menu-icon'
        });
        this._adjustIconSize();
        this.actor.add_actor(this._icon);
        this._notifyHeightId = this._panelActor.connect('notify::height', Lang.bind(this, function() {
            this._adjustIconSize();
        }));

        this._syncAppsButtonId = Main.overview.viewSelector._showAppsButton.connect('notify::checked', Lang.bind(this, function() {
            this._syncShowAppsButtonToggled();
        }));

        this._minHPadding = this._natHPadding = 0.0;
    },

    _adjustIconSize() {
        let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
        let iconSizes = baseMenuButtonIconSizes.map(function(s) {
            return s * scaleFactor;
        });

        let availSize = this._panelActor.get_height() - (MINIMUM_PADDING * 2);

        let newIconSize = baseMenuButtonIconSizes[0];
        for (let i = 0; i < iconSizes.length ; i++) {
            if (iconSizes[i] < availSize) {
                newIconSize = baseMenuButtonIconSizes[i];
            }
        }

        if (newIconSize == this._iconSize)
            return;

        let oldIconSize = this._iconSize;
        this._iconSize = newIconSize;
        this._icon.set_icon_size(this._iconSize);
    },

    _onStyleChanged(actor) {
        let themeNode = actor.get_theme_node();

        this._minHPadding = themeNode.get_length('-minimum-hpadding');
        this._natHPadding = themeNode.get_length('-natural-hpadding');
    },

    vfunc_get_preferred_width(forHeight) {
        let child = this.get_first_child();
        let minimumSize, naturalSize;

        if (child)
            [minimumSize, naturalSize] = child.get_preferred_width(-1);
        else
            minimumSize = naturalSize = 0;

        minimumSize += 2 * this._minHPadding;
        naturalSize += 2 * this._natHPadding;

        return [minimumSize, naturalSize];
    },

    vfunc_get_preferred_height(forWidth) {
        let child = this.get_first_child();

        if (child)
            return child.get_preferred_height(-1);

        return [0, 0];
    },

    vfunc_allocate(box, flags) {
        this.set_allocation(box, flags);

        let child = this.get_first_child();
        if (!child)
            return;

        let [minWidth, natWidth] = child.get_preferred_width(-1);

        let availWidth = box.x2 - box.x1;
        let availHeight = box.y2 - box.y1;

        let childBox = new Clutter.ActorBox();
        if (natWidth + 2 * this._natHPadding <= availWidth) {
            childBox.x1 = this._natHPadding;
            childBox.x2 = availWidth - this._natHPadding;
        } else {
            childBox.x1 = this._minHPadding;
            childBox.x2 = availWidth - this._minHPadding;
        }

        childBox.y1 = 0;
        childBox.y2 = availHeight;

        child.allocate(childBox, flags);
    },

    _onShowAppsButtonToggled() {
        // Sync the status of the default appButtons. Only if the two statuses are
        // different, that means the user interacted with the extension provided
        // application button, cutomize the behaviour. Otherwise the shell has changed the
        // status (due to the _syncShowAppsButtonToggled function below) and it
        // has already performed the desired action.

        let selector = Main.overview.viewSelector;

        if (selector._showAppsButton.checked !== this.showAppsButton.checked) {
            // find visible view
            let visibleView;
            Main.overview.viewSelector.appDisplay._views.every(function(v, index) {
                if (v.view.actor.visible) {
                    visibleView = index;
                    return false;
                }
                else
                    return true;
            });

            if (this.showAppsButton.checked) {
                // force spring animation triggering.By default the animation only
                // runs if we are already inside the overview.
                if (!Main.overview._shown) {
                    this.forcedOverview = true;
                    let view = Main.overview.viewSelector.appDisplay._views[visibleView].view;
                    let grid = view._grid;

                    // Animate in the the appview, hide the appGrid to avoiud flashing
                    // Go to the appView before entering the overview, skipping the workspaces.
                    // Do this manually avoiding opacity in transitions so that the setting of the opacity
                    // to 0 doesn't get overwritten.
                    Main.overview.viewSelector._activePage.opacity = 0;
                    Main.overview.viewSelector._activePage.hide();
                    Main.overview.viewSelector._activePage = Main.overview.viewSelector._appsPage;
                    Main.overview.viewSelector._activePage.show();
                    grid.actor.opacity = 0;

                    // The animation has to be trigered manually because the AppDisplay.animate
                    // method is waiting for an allocation not happening, as we skip the workspace view
                    // and the appgrid could already be allocated from previous shown.
                    // It has to be triggered after the overview is shown as wrong coordinates are obtained
                    // otherwise.
                    let overviewShownId = Main.overview.connect('shown', Lang.bind(this, function() {
                        Main.overview.disconnect(overviewShownId);
                        Meta.later_add(Meta.LaterType.BEFORE_REDRAW, Lang.bind(this, function() {
                            grid.actor.opacity = 255;
                            grid.animateSpring(IconGrid.AnimationDirection.IN, this.showAppsButton);
                        }));
                    }));
                }

                // Finally show the overview
                selector._showAppsButton.checked = true;
                Main.overview.show();
            }
            else {
                if (this.forcedOverview) {
                    // force exiting overview if needed

                    // Manually trigger springout animation without activating the
                    // workspaceView to avoid the zoomout animation. Hide the appPage
                    // onComplete to avoid ugly flashing of original icons.
                    let view = Main.overview.viewSelector.appDisplay._views[visibleView].view;
                    view.animate(IconGrid.AnimationDirection.OUT, Lang.bind(this, function() {
                        Main.overview.viewSelector._appsPage.hide();
                        Main.overview.hide();
                        selector._showAppsButton.checked = false;
                        this.forcedOverview = false;
                    }));
                }
                else {
                    selector._showAppsButton.checked = false;
                    this.forcedOverview = false;
                }
            }
        }
    },
    
    _syncShowAppsButtonToggled() {
        let status = Main.overview.viewSelector._showAppsButton.checked;
        if (this.showAppsButton.checked !== status)
            this.showAppsButton.checked = status;
    },

    _onDestroy() {
        this._panelActor.disconnect(this._notifyHeightId);
        Main.overview.viewSelector._showAppsButton.disconnect(this._syncAppsButtonId);
    }
});
